<#
.SYNOPSIS
	Show occurrences of communications with users in a specific federated SIP domain.

.DESCRIPTION
	Show occurrences of communications with users in a specific federated SIP domain. This can be crucial for domains where connectivity may be changing soon, such as the upcoming termination of PIC federation.

.NOTES
  Version      	   		: 1.0 - See changelog at http://www.ehloworld.com/2662
  Wish list						: Better error trapping
  Rights Required			: TBD
  Sched Task Required	: No
  Lync Server Version	: 2013
  Author(s)    				: Pat Richard
	Email/Blog/Twitter	: pat@innervation.com 	http://www.ehloworld.com @patrichard
	Dedicated Post			: http://www.ehloworld.com/2661
  Disclaimer   				: You running this script means you won't blame me if this breaks your stuff. This script is provided AS IS
												without warranty of any kind. I disclaim all implied warranties including, without limitation, any implied
												warranties of merchantability or of fitness for a particular purpose. The entire risk arising out of the use
												or performance of the sample scripts and documentation remains with you. In no event shall I be liable for
												any damages whatsoever (including, without limitation, damages for loss of business profits, business
												interruption, loss of business information, or other pecuniary loss) arising out of the use of or inability
												to use the script or documentation.
	Acknowledgements 		: Multiple values for switch: http://stackoverflow.com/questions/3493731/whats-the-powershell-syntax-for-multiple-values-in-a-switch-statement
											: Finding the first of the week: http://richardspowershellblog.wordpress.com/2008/03/27/start-and-end-of-week/
											: SQL query to find data: http://blogs.technet.com/b/rischwen/archive/2014/05/09/sql-query-find-federated-pic-conversations.aspx
	Assumptions					: ExecutionPolicy of AllSigned (recommended), RemoteSigned or Unrestricted (not recommended)
  Limitations					:
	Known issues				:

.EXAMPLE
	.\Get-CsFederatedConversationDetails.ps1 -SqlServer sql.contoso.com -SipDomain aol.com

	Description
	-----------
	Runs the script and queries SQL server specified specified for conversations with users at aol.com that have occurred this year.

.EXAMPLE
	.\Get-CsFederatedConversationDetails.ps1 -SqlServer sql.contoso.com -SipDomain aol.com -TimeSpan ThisMonth

	Description
	-----------
	Shows activitiy for the specified domain during the current year.
#>
#Requires -Version 2.0

[CmdletBinding(SupportsShouldProcess = $True)]
param(
	# Defines the SQL server to query
	[parameter(ValueFromPipeline = $false, ValueFromPipelineByPropertyName = $true)]
	[ValidateNotNullOrEmpty()]
	[string] $SqlServer = "cltsql2k8r203.apmetro.local",

	# If specified, defines the SQL instance to connect to
	[parameter(ValueFromPipeline = $false, ValueFromPipelineByPropertyName = $true)]
	[string] $SqlInstance,

	# The SIP domain to query for. All records involving SIP addresses at this SIP domain will be returned
	[parameter(ValueFromPipeline = $false, ValueFromPipelineByPropertyName = $true)]
	[ValidateNotNullOrEmpty()]
	[string] $SipDomain = "aol.com",

	# Time span in which to search. Defaults to this year.
	[parameter(ValueFromPipeline = $false, ValueFromPipelineByPropertyName = $true)]
	[ValidateNotNullOrEmpty()]
	$TimeSpan = "thisyear",

	# If specified, the end date to search for. All records for the specific SIP domain between $TimeSpan and $EndDate are returned.
	[parameter(ValueFromPipeline = $false, ValueFromPipelineByPropertyName = $true)]
	$EndDate
)
$month = Get-Date -Format MM
$year = Get-Date -Format yyyy

#region functions
function New-SqlQuery { # Modified version of http://blog.flaphead.com/2014/02/12/powershell-and-sql/
	[CmdletBinding(SupportsShouldProcess = $True)]
	param (
		# Defines the SQL server to connect to. Can also be specified as server\instance
		[Parameter(ValueFromPipeline = $True, ValueFromPipelineByPropertyName = $True)]
		[ValidateNotNullOrEmpty()]
		[string] $SQLserver,

		# Defines the database to connect to
		[Parameter(ValueFromPipeline = $True, ValueFromPipelineByPropertyName = $True)]
		[ValidateNotNullOrEmpty()]
		[string] $SQLdb,

		# Defines the query to process
		[Parameter(ValueFromPipeline = $True, ValueFromPipelineByPropertyName = $True)]
		[ValidateNotNullOrEmpty()]
		[string] $SQLcmd,

		# Defines the username used to connect to the SQL server
		[Parameter(ValueFromPipeline = $True, ValueFromPipelineByPropertyName = $True)]
		[string] $SQLUsername,

		# Defines the password used to connect to the SQL server.
		[Parameter(ValueFromPipeline = $True, ValueFromPipelineByPropertyName = $True)]
		[string] $SQLUserPassword
  )

  $Error.Clear()

  Write-Verbose "SQL Server   : $SQLserver"
  Write-Verbose "SQL Database : $SQLdb"
  Write-Verbose "SQL User     : $SQLUserName"
  Write-Verbose "SQL Password : $SQLUserPassword"
  Write-Verbose "Verbose      : $Verbose"
  Write-Verbose "SQLcmd       : $SQLcmd"

  $Connection = New-Object System.Data.SQLClient.SQLConnection
  if (!$SQLUserName){
    # Use Trusted Connection
    Write-Verbose "Using a trusted connection"
    $Connection.ConnectionString ="server=$SqlServer;database=$SqlDb;trusted_connection=true;"
  } else {
    # Use UserName and Password
    Write-Verbose "Using a specified username & password for connection"
    $Connection.ConnectionString ="server=$SqlServer;database=$SqlDb;User ID=$SQLUserName;Password=$SQLUserPassword;"
  }

  $Command = New-Object System.Data.SQLClient.SQLCommand
  $Command.CommandType = [System.Data.CommandType]"Text"
  $Command.Connection = $Connection
  If ($Error.Count -gt 0){Exit}
  $Command.CommandText = $SQLCmd
  $Command.CommandTimeout = 0

  $SqlData = New-Object "System.Data.DataTable"
  $Connection.Open()
  $Reader = $Command.ExecuteReader()
  $SqlData.Load($Reader)
  $Connection.Close()

  $i=0;$SqlData | ForEach{$i++}

  Write-Verbose "$i records returned"
  return $SqlData
} # end Function New-SqlQuery
#endregion functions

switch ($TimeSpan){
	LastWeek {
		# Last 7 days
		$StartDate = Get-Date ((Get-Date).AddDays(-7)) -Format MM-dd-yyyy
	}
	Last30Days {
		$StartDate = Get-Date ((Get-Date).AddDays(-30)) -Format MM-dd-yyyy
	}
	LastYear {
		# Last 365 days
		$StartDate = Get-Date ((Get-Date).AddDays(-365)) -Format MM-dd-yyyy
	}
	{($_ -eq "FirstOfYear") -or ($_ -eq "ThisYear")} {
		$StartDate = Get-Date 01-01-$year -Format yyyy-MM-dd
	}
	{($_ -eq "FirstOfMonth") -or ($_ -eq "ThisMonth")} {
		$StartDate = Get-Date $month-01-$year -Format yyyy-MM-dd
	}
	{($_ -eq "FirstOfWeek") -or ($_ -eq "ThisWeek")} {
		# Note: Starts with Sunday
		$s = Get-Date -Hour 0 -Minute 0 -Second 0
		$t = $s.AddDays(-($s).DayOfWeek.value__)
		$StartDate = Get-Date $t -Format yyyy-MM-dd
	}
	default {
		$StartDate = Get-Date $TimeSpan -Format yyyy-MM-dd
	}
} # end switch ($TimeSpan)

$Sql = "Select u1.UserUri User1URI, u2.UserUri User2URI, SessionIdTime From SessionDetails S inner join Users u1 on u1.UserId = S.User1Id inner join Users u2 on u2.UserId = S.User2Id Where (u1.UserUri like '%$SipDomain' or u2.UserUri like '%$SipDomain') and SessionIdTime >= '$StartDate'"
if ($EndDate){
	$EndDate = Get-Date (Get-Date $EndDate).AddDays(1) -Format yyyy-MM-dd
	$Sql += " and SessionIdTime !> '$EndDate'"
}

if ($SqlInstance){
	[string] $SQLServer += "\SQLInstance"
}

# Write-Host $sql -ForegroundColor Green

return New-SqlQuery -SQLserver "$SQLServer" -SQLdb "LcsCDR" -SQLcmd $Sql
# SIG # Begin signature block
# MIIQBQYJKoZIhvcNAQcCoIIP9jCCD/ICAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQUzB7cPnMA4jGE72MrYgTY1oTW
# ZG2ggg1KMIIGnzCCBYegAwIBAgIQCSu3nvOdPAJrDHL+XciEEzANBgkqhkiG9w0B
# AQUFADBvMQswCQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYD
# VQQLExB3d3cuZGlnaWNlcnQuY29tMS4wLAYDVQQDEyVEaWdpQ2VydCBBc3N1cmVk
# IElEIENvZGUgU2lnbmluZyBDQS0xMB4XDTEzMDgyODAwMDAwMFoXDTE0MDkwNDEy
# MDAwMFowazELMAkGA1UEBhMCVVMxCzAJBgNVBAgTAk1JMRkwFwYDVQQHExBTdGVy
# bGluZyBIZWlnaHRzMRkwFwYDVQQKExBJbm5lcnZhdGlvbiwgTExDMRkwFwYDVQQD
# ExBJbm5lcnZhdGlvbiwgTExDMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKC
# AQEA8dXC+TSUZsMisaialKJLat7AnmOF0HE+U49duaBJp/N4IXhjUZM9sXRkR/X0
# KAbNauNpNrs3TLKNpO7/RUinzrzsg8IhtBVXSrK7c0HDqJ7vG181it2qQO1yB9uD
# UV8frjprnNcR1w0qW9wFqclKlkyOxgiCANVoT6qXO9IxPZT28h+tMfBCsU7sZALs
# 3kPRMCQ1ZywngVSqEQ9rBYV3jW4EgvPEOmuhtnwgVB1Alr2V1dVAtbNxw1A7fQ8w
# J6EQSNAPVJQq7304QKKgGaTXtA2TwAjO5+m9ZltD/s0wXWYs0V1r8pOJOo1r17l4
# z91OWLPP+yfxvlnm5F1IXZxCbQIDAQABo4IDOTCCAzUwHwYDVR0jBBgwFoAUe2jO
# KarAF75JeuHlP9an90WPNTIwHQYDVR0OBBYEFB9N9j4QNDPcSscWMAVDKCOcXS/G
# MA4GA1UdDwEB/wQEAwIHgDATBgNVHSUEDDAKBggrBgEFBQcDAzBzBgNVHR8EbDBq
# MDOgMaAvhi1odHRwOi8vY3JsMy5kaWdpY2VydC5jb20vYXNzdXJlZC1jcy0yMDEx
# YS5jcmwwM6AxoC+GLWh0dHA6Ly9jcmw0LmRpZ2ljZXJ0LmNvbS9hc3N1cmVkLWNz
# LTIwMTFhLmNybDCCAcQGA1UdIASCAbswggG3MIIBswYJYIZIAYb9bAMBMIIBpDA6
# BggrBgEFBQcCARYuaHR0cDovL3d3dy5kaWdpY2VydC5jb20vc3NsLWNwcy1yZXBv
# c2l0b3J5Lmh0bTCCAWQGCCsGAQUFBwICMIIBVh6CAVIAQQBuAHkAIAB1AHMAZQAg
# AG8AZgAgAHQAaABpAHMAIABDAGUAcgB0AGkAZgBpAGMAYQB0AGUAIABjAG8AbgBz
# AHQAaQB0AHUAdABlAHMAIABhAGMAYwBlAHAAdABhAG4AYwBlACAAbwBmACAAdABo
# AGUAIABEAGkAZwBpAEMAZQByAHQAIABDAFAALwBDAFAAUwAgAGEAbgBkACAAdABo
# AGUAIABSAGUAbAB5AGkAbgBnACAAUABhAHIAdAB5ACAAQQBnAHIAZQBlAG0AZQBu
# AHQAIAB3AGgAaQBjAGgAIABsAGkAbQBpAHQAIABsAGkAYQBiAGkAbABpAHQAeQAg
# AGEAbgBkACAAYQByAGUAIABpAG4AYwBvAHIAcABvAHIAYQB0AGUAZAAgAGgAZQBy
# AGUAaQBuACAAYgB5ACAAcgBlAGYAZQByAGUAbgBjAGUALjCBggYIKwYBBQUHAQEE
# djB0MCQGCCsGAQUFBzABhhhodHRwOi8vb2NzcC5kaWdpY2VydC5jb20wTAYIKwYB
# BQUHMAKGQGh0dHA6Ly9jYWNlcnRzLmRpZ2ljZXJ0LmNvbS9EaWdpQ2VydEFzc3Vy
# ZWRJRENvZGVTaWduaW5nQ0EtMS5jcnQwDAYDVR0TAQH/BAIwADANBgkqhkiG9w0B
# AQUFAAOCAQEABTcbxhPxSJP7AKrF23wXbgelEGzjpewmm0BnHO9/V+9K4UE/3i5Z
# e3bX7OvUb/55zCHT4pvZl8/i8tuvFnrWZZcjGljT3W9SIVOuKNdlItXC8dvP67Hz
# OMZvSbdI5MU6sjd3vyYKznTMHG1Mz/x+BMh6rEnGsIaY4r8r4fkNfJTyTkww8AWh
# I16Pno3dWgKhfploPODtMxczJKA15VDFhQqM3oHHMojasXo5uGb5jlGiDFkZ3mDa
# /+gLUsJUjAzF27ZIy/jRgp0eMkqlgdxGa+gZUK73Ue6aTP2LW9SL/1fLYl8QkwRs
# MWG1faGdureT3Oh0EHH6UhREB/SxLJ+xLTCCBqMwggWLoAMCAQICEA+oSQYV1wCg
# viF2/cXsbb0wDQYJKoZIhvcNAQEFBQAwZTELMAkGA1UEBhMCVVMxFTATBgNVBAoT
# DERpZ2lDZXJ0IEluYzEZMBcGA1UECxMQd3d3LmRpZ2ljZXJ0LmNvbTEkMCIGA1UE
# AxMbRGlnaUNlcnQgQXNzdXJlZCBJRCBSb290IENBMB4XDTExMDIxMTEyMDAwMFoX
# DTI2MDIxMDEyMDAwMFowbzELMAkGA1UEBhMCVVMxFTATBgNVBAoTDERpZ2lDZXJ0
# IEluYzEZMBcGA1UECxMQd3d3LmRpZ2ljZXJ0LmNvbTEuMCwGA1UEAxMlRGlnaUNl
# cnQgQXNzdXJlZCBJRCBDb2RlIFNpZ25pbmcgQ0EtMTCCASIwDQYJKoZIhvcNAQEB
# BQADggEPADCCAQoCggEBAJx8+aCPCsqJS1OaPOwZIn8My/dIRNA/Im6aT/rO38bT
# JJH/qFKT53L48UaGlMWrF/R4f8t6vpAmHHxTL+WD57tqBSjMoBcRSxgg87e98tzL
# uIZARR9P+TmY0zvrb2mkXAEusWbpprjcBt6ujWL+RCeCqQPD/uYmC5NJceU4bU7+
# gFxnd7XVb2ZklGu7iElo2NH0fiHB5sUeyeCWuAmV+UuerswxvWpaQqfEBUd9YCvZ
# oV29+1aT7xv8cvnfPjL93SosMkbaXmO80LjLTBA1/FBfrENEfP6ERFC0jCo9dAz0
# eotyS+BWtRO2Y+k/Tkkj5wYW8CWrAfgoQebH1GQ7XasCAwEAAaOCA0MwggM/MA4G
# A1UdDwEB/wQEAwIBhjATBgNVHSUEDDAKBggrBgEFBQcDAzCCAcMGA1UdIASCAbow
# ggG2MIIBsgYIYIZIAYb9bAMwggGkMDoGCCsGAQUFBwIBFi5odHRwOi8vd3d3LmRp
# Z2ljZXJ0LmNvbS9zc2wtY3BzLXJlcG9zaXRvcnkuaHRtMIIBZAYIKwYBBQUHAgIw
# ggFWHoIBUgBBAG4AeQAgAHUAcwBlACAAbwBmACAAdABoAGkAcwAgAEMAZQByAHQA
# aQBmAGkAYwBhAHQAZQAgAGMAbwBuAHMAdABpAHQAdQB0AGUAcwAgAGEAYwBjAGUA
# cAB0AGEAbgBjAGUAIABvAGYAIAB0AGgAZQAgAEQAaQBnAGkAQwBlAHIAdAAgAEMA
# UAAvAEMAUABTACAAYQBuAGQAIAB0AGgAZQAgAFIAZQBsAHkAaQBuAGcAIABQAGEA
# cgB0AHkAIABBAGcAcgBlAGUAbQBlAG4AdAAgAHcAaABpAGMAaAAgAGwAaQBtAGkA
# dAAgAGwAaQBhAGIAaQBsAGkAdAB5ACAAYQBuAGQAIABhAHIAZQAgAGkAbgBjAG8A
# cgBwAG8AcgBhAHQAZQBkACAAaABlAHIAZQBpAG4AIABiAHkAIAByAGUAZgBlAHIA
# ZQBuAGMAZQAuMBIGA1UdEwEB/wQIMAYBAf8CAQAweQYIKwYBBQUHAQEEbTBrMCQG
# CCsGAQUFBzABhhhodHRwOi8vb2NzcC5kaWdpY2VydC5jb20wQwYIKwYBBQUHMAKG
# N2h0dHA6Ly9jYWNlcnRzLmRpZ2ljZXJ0LmNvbS9EaWdpQ2VydEFzc3VyZWRJRFJv
# b3RDQS5jcnQwgYEGA1UdHwR6MHgwOqA4oDaGNGh0dHA6Ly9jcmwzLmRpZ2ljZXJ0
# LmNvbS9EaWdpQ2VydEFzc3VyZWRJRFJvb3RDQS5jcmwwOqA4oDaGNGh0dHA6Ly9j
# cmw0LmRpZ2ljZXJ0LmNvbS9EaWdpQ2VydEFzc3VyZWRJRFJvb3RDQS5jcmwwHQYD
# VR0OBBYEFHtozimqwBe+SXrh5T/Wp/dFjzUyMB8GA1UdIwQYMBaAFEXroq/0ksuC
# MS1Ri6enIZ3zbcgPMA0GCSqGSIb3DQEBBQUAA4IBAQB7ch1k/4jIOsG36eepxIe7
# 25SS15BZM/orh96oW4AlPxOPm4MbfEPE5ozfOT7DFeyw2jshJXskwXJduEeRgRNG
# +pw/alE43rQly/Cr38UoAVR5EEYk0TgPJqFhkE26vSjmP/HEqpv22jVTT8nyPdNs
# 3CPtqqBNZwnzOoA9PPs2TJDndqTd8jq/VjUvokxl6ODU2tHHyJFqLSNPNzsZlBjU
# 1ZwQPNWxHBn/j8hrm574rpyZlnjRzZxRFVtCJnJajQpKI5JA6IbeIsKTOtSbaKbf
# KX8GuTwOvZ/EhpyCR0JxMoYJmXIJeUudcWn1Qf9/OXdk8YSNvosesn1oo6WQsQz/
# MYICJTCCAiECAQEwgYMwbzELMAkGA1UEBhMCVVMxFTATBgNVBAoTDERpZ2lDZXJ0
# IEluYzEZMBcGA1UECxMQd3d3LmRpZ2ljZXJ0LmNvbTEuMCwGA1UEAxMlRGlnaUNl
# cnQgQXNzdXJlZCBJRCBDb2RlIFNpZ25pbmcgQ0EtMQIQCSu3nvOdPAJrDHL+XciE
# EzAJBgUrDgMCGgUAoHgwGAYKKwYBBAGCNwIBDDEKMAigAoAAoQKAADAZBgkqhkiG
# 9w0BCQMxDAYKKwYBBAGCNwIBBDAcBgorBgEEAYI3AgELMQ4wDAYKKwYBBAGCNwIB
# FTAjBgkqhkiG9w0BCQQxFgQUQmiLz/bFQarC36nX4+9LcZylhL4wDQYJKoZIhvcN
# AQEBBQAEggEAtMNpZP5mWQMZkdntvdd2UlT6nvheacaTMj7m9FIw3sIZnK+kbhzI
# 3V/jfQjbUaIuTB1U7IKSRle/fxb3Z8o0AjPt0jDtw6bNKO6On2VkuiHf5RhqmFEY
# MiPku04i5K73mTp28/fA1YyRkBe6wQ3NRy/S9uxAhAUNN2Ver0g1OYPVTtrjW99o
# hRdi95qFHLqQAscpCfrvTGuccKRhoI7glXj8r/6GzGJwI24S2UcXqNp7LZ8Iv/Sj
# yMunwqXY2Fnjij10MlZJ62Nf4OkikUpoa54ENCT5PEe5TNT5eYuJW8TbNgOxzVm2
# iI3A6JS4oNbMLiIEF1LDb2h2aQoKDdS9jg==
# SIG # End signature block
